// Copyright (C) 2009 Playedit.

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef TYPES_H
#define TYPES_H

#include "config.h"

#include <glibmm/object.h>
#include <gdkmm/pixbuf.h>
#include <stdint.h>

typedef struct LIBMTP_album_struct LIBMTP_album_t;
typedef struct LIBMTP_track_struct LIBMTP_track_t;
typedef struct LIBMTP_playlist_struct LIBMTP_playlist_t;
typedef struct LIBMTP_filesampledata_struct LIBMTP_filesampledata_t;

class mtp_data_t : public virtual Glib::Object
{
	public:
		virtual Glib::ustring Name() const = 0;
		virtual ~mtp_data_t() {};

	protected:
		mtp_data_t() : Glib::ObjectBase(typeid(mtp_data_t)), Glib::Object() {};
};

class track_t;

class album_t : public mtp_data_t
{
	public:
		friend class MTPDevice;

		static Glib::RefPtr<album_t> create(LIBMTP_album_t* _album = NULL);
		~album_t();

		Glib::ustring RawName() const;
		Glib::ustring Name() const;
		Glib::ustring Artist() const;
		Glib::ustring RawArtist() const;
		Glib::ustring Genre() const;
		uint32_t Id() const;
		uint32_t NumTracks() const;
		std::vector<uint32_t> Tracks() const;

		void SetName(const Glib::ustring& name);
		void SetArtist(const Glib::ustring& artist);
		void SetGenre(const Glib::ustring& genre);
		void SetTracks(std::vector<uint32_t> tracks);

		Glib::RefPtr<Gdk::Pixbuf> art;

	private:
		album_t(LIBMTP_album_t* _album = NULL);

		LIBMTP_album_t* album;
};

typedef std::pair< Glib::RefPtr<album_t>, std::vector< Glib::RefPtr<track_t> > > album_pair_t;

class track_t : public mtp_data_t
{
	public:
		friend class MTPDevice;

		static Glib::RefPtr<track_t> create(LIBMTP_track_t* _track = NULL);
		~track_t();

		Glib::ustring Name() const;
		Glib::ustring Album() const;
		Glib::ustring RawAlbum() const;
		Glib::ustring Artist() const;
		Glib::ustring RawArtist() const;
		Glib::ustring Composer() const;
		Glib::ustring Genre() const;
		Glib::ustring Date() const;
		uint32_t Duration() const;
		uint16_t TrackNumber() const;
		uint32_t Id() const;
		uint16_t Rating() const;
		uint32_t UseCount() const;

		void SetFileType(const std::string& filetype);
		void SetFileName(const std::string& filename);
		void SetFileSize(const uint64_t size);
		void SetTitle(const Glib::ustring& title);
		void SetArtist(const Glib::ustring& artist);
		void SetAlbum(const Glib::ustring& album);
		void SetGenre(const Glib::ustring& genre);
		void SetDate(const Glib::ustring& date);
		void SetTrackNumber(uint16_t track_num);
		void SetNumChannels(uint16_t num_channels);
		void SetDuration(uint32_t duration);

		Glib::RefPtr<Gdk::Pixbuf> art;

	private:
		track_t(LIBMTP_track_t* _track = NULL);

		LIBMTP_track_t* track;
};

class playlist_t : public mtp_data_t
{
	public:
		friend class MTPDevice;

		static Glib::RefPtr<playlist_t> create(LIBMTP_playlist_t* _playlist = NULL);
		~playlist_t();

		Glib::ustring Name() const;
		uint32_t Id() const;
		uint32_t NumTracks() const;
		std::vector<uint32_t> Tracks() const;

		void SetName(const Glib::ustring& name);
		void SetTracks(std::vector<uint32_t> tracks);

	private:
		playlist_t(LIBMTP_playlist_t* _playlist = NULL);

		LIBMTP_playlist_t* playlist;
};

class albumart_t
{
	public:
		friend class MTPDevice;

		albumart_t();
		~albumart_t();

	private:
		LIBMTP_filesampledata_t* data;
};

int AlbumCompareNameFirst(const Glib::RefPtr<album_t>& a,
		const Glib::RefPtr<album_t>& b);
int AlbumCompareArtistFirst(const Glib::RefPtr<album_t>& a,
		const Glib::RefPtr<album_t>& b);
int TrackCompare(const Glib::RefPtr<const track_t>& a,
		const Glib::RefPtr<const track_t>& b);
bool PlaylistIsLess(const Glib::RefPtr<const playlist_t>& a,
		const Glib::RefPtr<const playlist_t>& b);

#endif // TYPES_H
