// Copyright (C) 2009 Playedit.

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "config.h"

#include "mtp_device.h"
#include "music.h"
#include "id3reader.h"
#include "album_selector.h"

#include <glibmm/i18n.h>
#include <giomm/file.h>
#include <iostream>

bool MusicView::on_drag_motion(const Glib::RefPtr<Gdk::DragContext>& context,
		int x, int y, guint time)
{
	if (!m_App.m_Device.IsReady())
	{
		context->drag_refuse(time);
		return false;
	}

	return Gtk::TreeView::on_drag_motion(context, x, y, time);
}

void MusicView::on_drag_data_received(const Glib::RefPtr<Gdk::DragContext>& context,
		int /* x */, int /* y */, const Gtk::SelectionData& selection_data, guint /* info */, guint time)
{
	std::vector<Glib::ustring> fileuris = selection_data.get_uris();
	for (std::vector<Glib::ustring>::iterator i = fileuris.begin();
			i < fileuris.end(); ++i)
	{
		Glib::RefPtr<Gio::File> file = Gio::File::create_for_uri(*i);

		// TODO: Add other filetypes
		if (file && file->query_info()->get_content_type() == "audio/mpeg")
		{
			// Create the track and set its metadata using available id3 tags
			Glib::RefPtr<track_t> track = track_t::create();

			track->SetFileType(file->query_info()->get_content_type());
			track->SetFileName(file->query_info()->get_name());
			track->SetFileSize(file->query_info()->get_size());

			ID3TagReader id3_reader(file->get_path());
			if (id3_reader.Load())
			{
				track->SetTitle(id3_reader.GetTitle());
				track->SetArtist(id3_reader.GetArtist());
				track->SetAlbum(id3_reader.GetAlbum());
				track->SetGenre(id3_reader.GetGenre());
				track->SetDate(id3_reader.GetYear());
				track->SetTrackNumber(id3_reader.GetTrackNumber());
				track->SetNumChannels(id3_reader.GetNumChannels());
				track->SetDuration(id3_reader.GetDuration());
			}

			// Send the track to the device
			m_App.SetStatus(_("Sending track to device..."));
			if (m_App.m_Device.CreateNewTrack(file->get_path(), track))
				FindAlbumAndAddTrack(track);
			m_App.UnsetStatus();
		}
	}
	context->drag_finish(true, false, time);
}
